﻿using HeuristicLab.Services.Deployment;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using HeuristicLab.Services.Deployment.DataAccess;
using System;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Diagnostics;
using System.Collections.Generic;

namespace HeuristicLab.Services.Deployment.Test {


  /// <summary>
  ///This is a test class for AdminTest and is intended
  ///to contain all AdminTest Unit Tests
  ///</summary>
  [TestClass()]
  public class AdminTest {

    private Encoding enc = new ASCIIEncoding();
    private TestContext testContextInstance;

    /// <summary>
    ///Gets or sets the test context which provides
    ///information about and functionality for the current test run.
    ///</summary>
    public TestContext TestContext {
      get {
        return testContextInstance;
      }
      set {
        testContextInstance = value;
      }
    }

    #region Additional test attributes
    // 
    //You can use the following additional attributes as you write your tests:
    //
    //Use ClassInitialize to run code before running the first test in the class
    //[ClassInitialize()]
    //public static void MyClassInitialize(TestContext testContext)
    //{
    //}
    //
    //Use ClassCleanup to run code after all tests in a class have run
    //[ClassCleanup()]
    //public static void MyClassCleanup()
    //{
    //}
    //
    //Use TestInitialize to run code before running each test
    //[TestInitialize()]
    //public void MyTestInitialize()
    //{
    //}
    //
    //Use TestCleanup to run code after each test has run
    //[TestCleanup()]
    //public void MyTestCleanup()
    //{
    //}
    //
    #endregion


    /// <summary>
    ///A test for StoreProduct
    ///</summary>
    [TestMethod()]
    public void DeployProductTest() {
      var target = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
      var update = new HeuristicLab.Services.Deployment.Test.UpdateService.UpdateClient();
      int oldCount = update.GetProducts().Count();

      string prodName = RandomName();
      Version vers01 = new Version("0.0.0.1");
      ProductDescription product = new ProductDescription(prodName, vers01);
      target.DeployProduct(product);

      int newCount = update.GetProducts().Count();
      Assert.AreEqual(oldCount + 1, newCount);
    }

    /// <summary>
    ///A test for DeployPlugin
    ///</summary>
    [TestMethod()]
    public void DeployPluginTest() {
      var target = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
      var update = new HeuristicLab.Services.Deployment.Test.UpdateService.UpdateClient();

      var vers01 = new Version(0, 1);

      #region persist single plugin without dependencies
      {
        string name = RandomName();
        int oldCount = update.GetPlugins().Count();
        PluginDescription pluginDescription = new PluginDescription(name, vers01);
        byte[] pluginPackage = enc.GetBytes("Zipped " + name);
        target.DeployPlugin(pluginDescription, pluginPackage);
        int newCount = update.GetPlugins().Count();
        Assert.AreEqual(oldCount + 1, newCount);
      }
      #endregion

      #region persist a product with same name and version as an existent product
      {
        string name = RandomName();
        int oldCount = update.GetPlugins().Count();
        PluginDescription pluginDescription = new PluginDescription(name, vers01);
        byte[] pluginPackage = enc.GetBytes("Zipped " + name);
        target.DeployPlugin(pluginDescription, pluginPackage);
        int newCount = update.GetPlugins().Count();
        Assert.AreEqual(oldCount + 1, newCount);

        // insert same name and version
        oldCount = update.GetPlugins().Count();
        pluginDescription = new PluginDescription(name, vers01);
        pluginPackage = enc.GetBytes("Zipped " + name);
        target.DeployPlugin(pluginDescription, pluginPackage);
        newCount = update.GetPlugins().Count();
        // make sure old entry was updated
        Assert.AreEqual(oldCount, newCount);

        // insert new with different version
        oldCount = update.GetPlugins().Count();
        pluginDescription = new PluginDescription(name, new Version(0, 2));
        pluginPackage = enc.GetBytes("Zipped " + name);
        target.DeployPlugin(pluginDescription, pluginPackage);
        newCount = update.GetPlugins().Count();
        // make sure a new entry was created
        Assert.AreEqual(oldCount + 1, newCount);
      }
      #endregion

      #region persist a plugin with an already persisted dependency
      {
        string name = RandomName();
        PluginDescription dependency = new PluginDescription(name, vers01);
        // insert dependency first
        target.DeployPlugin(dependency, enc.GetBytes("Zipped " + name));

        // persist another plugin that has a dependency on the first plugin
        string name2 = RandomName();
        int oldCount = update.GetPlugins().Count();

        PluginDescription newEntity = new PluginDescription(name2, vers01, Enumerable.Repeat(dependency, 1));
        byte[] newEntityPackage = enc.GetBytes("Zipped " + name2);
        target.DeployPlugin(newEntity, newEntityPackage);
        int newCount = update.GetPlugins().Count();
        Assert.AreEqual(oldCount + 1, newCount); // only one new plugin should be added

        // retrieve second plugin and check dependencies
        newEntity = update.GetPlugins().Where(p => p.Name == name2).Single();
        Assert.AreEqual(newEntity.Dependencies.Count(), 1);
        Assert.AreEqual(newEntity.Dependencies.First().Name, name);
      }
      #endregion

      #region try to persist a new  plugin with a non-existant dependency and check the expected exception
      {
        // try to persist a new plugin with a non-existant dependency
        try {
          string pluginName = RandomName();
          string dependencyName = RandomName();
          var dependency = new PluginDescription(dependencyName, vers01);
          var newEntity = new PluginDescription(pluginName, vers01, Enumerable.Repeat(dependency, 1));
          target.DeployPlugin(newEntity, enc.GetBytes("Zipped " + pluginName));
          Assert.Fail("persist should fail with ArgumentException");
        }
        catch (FaultException e) {
          // this is expected
          Assert.IsTrue(true, "expected exception");

          // reconnect service
          target = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
        }
      }
      #endregion

      #region update the plugin file of an existing plugin
      {
        // insert new plugin
        string pluginName = RandomName();
        var newPlugin = new PluginDescription(pluginName, vers01);
        target.DeployPlugin(newPlugin, enc.GetBytes("Zipped " + pluginName));

        // update the plugin file
        byte[] expected = enc.GetBytes("Zipped2 " + pluginName);
        target.DeployPlugin(newPlugin, expected);
        // check if the updated file is returned
        byte[] actual = update.GetPlugin(newPlugin);
        // check retrieved file
        Assert.AreEqual(expected.Length, actual.Length);
        for (int i = 0; i < expected.Length; i++) {
          Assert.AreEqual(expected[i], actual[i]);
        }
      }
      #endregion

      #region update the dependencies of an existing plugin
      {
        string dependency1Name = RandomName();
        var newDependency1 = new PluginDescription(dependency1Name, vers01);
        target.DeployPlugin(newDependency1, enc.GetBytes("Zipped " + dependency1Name));

        string pluginName = RandomName();
        var newPlugin = new PluginDescription(pluginName, vers01, Enumerable.Repeat(newDependency1, 1));
        target.DeployPlugin(newPlugin, enc.GetBytes("Zipped " + pluginName));

        // retrieve plugin
        var dbPlugin = update.GetPlugins().Where(p => p.Name == pluginName).Single();
        Assert.AreEqual(dbPlugin.Dependencies.Count(), 1);
        Assert.AreEqual(dbPlugin.Dependencies.First().Name, dependency1Name);

        // change dependencies 
        string dependency2Name = RandomName();
        var newDependency2 = new PluginDescription(dependency2Name, vers01);
        target.DeployPlugin(newDependency2, enc.GetBytes("Zipped " + pluginName));

        newPlugin = new PluginDescription(pluginName, vers01, new PluginDescription[] { newDependency1, newDependency2 });
        target.DeployPlugin(newPlugin, enc.GetBytes("Zipped " + pluginName));
        // retrieve plugin
        dbPlugin = update.GetPlugins().Where(p => p.Name == pluginName).Single();
        Assert.AreEqual(dbPlugin.Dependencies.Count(), 2);
      }
      #endregion

      #region try to insert a plugin that references the same dependency twice and check if the correct exception is thrown
      {
        string depName = RandomName();
        var depPlugin = new PluginDescription(depName, vers01);
        target.DeployPlugin(depPlugin, enc.GetBytes("Zipped " + depName));

        // insert new plugin
        string pluginName = RandomName();
        var plugin = new PluginDescription(pluginName, vers01, new PluginDescription[] { depPlugin, depPlugin });
        try {
          target.DeployPlugin(plugin, enc.GetBytes("Zipped " + depName));
          Assert.Fail("Expected ArgumentException");
        }
        catch (FaultException) {
          Assert.IsTrue(true, "Exception thrown as expected");
          // reconnect
          target = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
        }
      }
      #endregion

      #region try to insert a plugin that with a cyclic reference
      {
        string depName = RandomName();
        var depPlugin = new PluginDescription(depName, vers01);
        target.DeployPlugin(depPlugin, enc.GetBytes("Zipped " + depName));

        // update the plugin so that it has a dependency on itself 
        var plugin = new PluginDescription(depName, vers01, new PluginDescription[] { depPlugin });
        try {
          target.DeployPlugin(plugin, enc.GetBytes("Zipped " + depName));
          Assert.Fail("Expected ArgumentException");
        }
        catch (FaultException) {
          target = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
          Assert.IsTrue(true, "Exception thrown as expected");
        }
      }
      #endregion
    }

    [TestMethod]
    public void DeployPluginPerformanceTest() {
      int nPlugins = 100;
      int maxDependencies = 10;
      int avgZipFileLength = 32000;

      var adminClient = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
      var updateClient = new HeuristicLab.Services.Deployment.Test.UpdateService.UpdateClient();
      Version vers01 = new Version(0, 1);
      // create a random byte array to represent file length
      byte[] zippedConstant = new byte[avgZipFileLength];
      Random r = new Random();
      r.NextBytes(zippedConstant);

      Stopwatch stopWatch = new Stopwatch();
      stopWatch.Start();
      // create plugins
      for (int i = 0; i < nPlugins; i++) {
        string name = RandomName();
        var allPlugins = updateClient.GetPlugins();
        if (allPlugins.Length > maxDependencies) {
          PluginDescription[] dependencies = new PluginDescription[maxDependencies];
          for (int j = 0; j < maxDependencies; j++) {
            dependencies[j] = allPlugins[j];
          }
          var plugin = new PluginDescription(name, vers01, dependencies);
          adminClient.DeployPlugin(plugin, zippedConstant);
        } else {
          var plugin = new PluginDescription(name, vers01, allPlugins);
          adminClient.DeployPlugin(plugin, zippedConstant);
        }
      }
      stopWatch.Stop();
      Assert.Inconclusive("Created " + nPlugins + " plugins in " + stopWatch.ElapsedMilliseconds +
        " ms (" + (double)stopWatch.ElapsedMilliseconds / nPlugins + " ms / plugin )");
    }
    [TestMethod]
    public void DeployProductPerformanceTest() {
      int nProducts = 50;
      int avgProductPlugins = 30;

      var adminClient = new HeuristicLab.Services.Deployment.Test.AdminService.AdminClient();
      var updateClient = new HeuristicLab.Services.Deployment.Test.UpdateService.UpdateClient();
      Version vers01 = new Version(0, 1);
      Stopwatch stopWatch = new Stopwatch();
      Random r = new Random();
      var plugins = updateClient.GetPlugins().ToList();
      stopWatch.Start();
      // create products
      for (int i = 0; i < nProducts; i++) {
        string name = RandomName();
        List<PluginDescription> prodPlugins = new List<PluginDescription>();
        for (int j = 0; j < avgProductPlugins; j++) {
          var selectedPlugin = plugins[r.Next(0, plugins.Count)];
          if (!prodPlugins.Contains(selectedPlugin)) prodPlugins.Add(selectedPlugin);
        }
        var prod = new ProductDescription(name, vers01, prodPlugins);
        adminClient.DeployProduct(prod);
      }
      stopWatch.Stop();
      Assert.Inconclusive("Created " + nProducts + " products in " + stopWatch.ElapsedMilliseconds +
        " ms (" + (double)stopWatch.ElapsedMilliseconds / nProducts + " ms / product )");
    }

    private string RandomName() {
      return Guid.NewGuid().ToString();
    }
  }
}
